import { Meteor } from 'meteor/meteor';
import { Mongo } from 'meteor/mongo';
import { check } from 'meteor/check';

import { Tasks } from '../models/task.js';

Meteor.methods({
  'tasks.insert'(task) {
    check(task.name, String);

    // Create the record.
    Tasks.insert({
      name: task.name,
      isGroupTask : task.isGroupTask,
      score : task.score,
      taskIconIndex : task.iconIndex,
      personId : task.personId,
      age : task.age
    });

  },
  'tasks.assignTo'(taskId, personId, clientId){
    var task = Tasks.findOne(taskId);
    if (task.personId === undefined){
      // clone taks
      Tasks.insert({
        name: task.name,
        isGroupTask : task.isGroupTask,
        score : task.score,
        taskIconIndex : task.taskIconIndex,
        personId : personId,
        age : task.age,
        isAssigned : true,
        clientId : clientId
      });
    }
    else  {
      Tasks.update(taskId, { $set: { personId: personId } });
    }
  },
  'tasks.isApproved'(taskId){
    var task = Tasks.findOne(taskId);
    Tasks.update(taskId, { $set: { isApproved: true } });
  },
  'tasks.isCompleted'(taskId, setIsCompleted){
    Tasks.update(taskId, { $set: { isCompleted: setIsCompleted } });
  },
  'tasks.remove'(taskId) {
    Tasks.remove({ _id : taskId});
  },
  'tasks.removeAll'() {
    Tasks.remove({});
  },
  'tasks.addAttributes'(id, fields) {
    Tasks.update(
      {
        _id: id
      },
      {
        $set: fields
      });
  },
  'tasks.removeAttributes'(name, fields) {
    Tasks.update(
      {
        name: name
      },
      {
        $unset: fields
      });
  },
  'tasks.get'() {
    return Tasks.find({});
  }
});

if (Meteor.isServer) {
  Meteor.publish('tasks', function() {
    return Tasks.find({ isAssigned : {$ne: true}});
  });
  Meteor.publish('taskInstances', function() {
    return Tasks.find({ isAssigned : {$eq: true}, isGoaled : { $ne : true } });
  });
}

Meteor.startup(() => {
  if (Tasks.find({}).count() === 0) {
    Tasks.insert({ name : "Lage middag", score : 5, age : 5, taskIconIndex : 11 });
    Tasks.insert({ name : "Gå ut med søppel", score : 5, age : 5, taskIconIndex : 1 });
    Tasks.insert({ name : "Henge klær til tørk", score : 5, age : 4, taskIconIndex : 4 });    
  }
});
