import { Meteor } from 'meteor/meteor';
import { Mongo } from 'meteor/mongo';
import { check } from 'meteor/check';

export const Profiles = new Mongo.Collection('profiles');

Profiles.attachSchema(new SimpleSchema({
  name: {
    type: String,
  },
  birthDate: {
    type: Date,
  },
  groupId: {
    type: String,
  },
  color: {
    type: String,
    allowedValues: [
      '#ff9900',
      '#ff9900',
      '#ff0000',
    ],
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert && !this.isSet) {
        return Profiles.simpleSchema()._schema.color.allowedValues[0];
      }
    },
  },
  createdAt: {
    type: Date,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert) {
        return new Date();
      } else {
        this.unset();
      }
    },
  },
  modifiedAt: {
    type: Date,
    optional: true,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isUpdate) {
        return new Date();
      } else {
        this.unset();
      }
    },
  },
  'avatar': {
    type: Object,
    autoValue() { // eslint-disable-line consistent-return
      if (this.isInsert && !this.isSet) {
        return {
          colorIndex: 1,
          iconIndex: 2
        };
      }
    },
  },
  'avatar.colorIndex': {
    type: Number,
  },
  'avatar.iconIndex': {
    type: Number,
  }
}));