import React, { Component } from 'react';
import ReactDOM from 'react-dom';
import { createContainer } from 'meteor/react-meteor-data'; 
import { Profiles } from '../models/profile.js';
import { Icons } from '../models/icon.js';
import Profile from './Profile.jsx';
import moment from 'moment';
 

 class ProfileList extends Component { 
constructor(props) {
    super(props);
 
    this.state = {
      adding: false,
      selectedIndex:0,
      selectedColorIndex:0,
      months: ["januar","februar","mars","april","mai","juni","juli","august","september","oktober","november","desember"],
      years: this.getYears(),
      age:0
    };
    
  }
 handleSubmit(event){
   event.preventDefault();
   
   const target = event.target;
   const profile = {};
   const groupId = this.props.family._id;
   profile.name = target.addtaskTitle.value;
   profile.birthDate = new Date(this.state.selectedYear, this.state.selectedMonth,this.state.selectedDay);
   profile.colorIndex = this.state.selectedColorIndex;
   profile.iconIndex = this.state.selectedIndex;
   
   Meteor.call('profile.insert', groupId, profile);
   this.setState({adding:false, selectedIndex: 0, selectedColorIndex:0});
 }
yearSelected(event){
this.setState({selectedYear: event.target.value, age: this.calculateAge() }); 


}
monthSelected(event){
this.setState({selectedMonth: parseInt(event.target.value), age: this.calculateAge() }); 


}

daySelected(event){
this.setState({selectedDay: event.target.value, age: this.calculateAge()}); 

}

 getYears(){
   
   let currentYear = new Date(Date.now()).getFullYear();
   let years = [];
  for(let i = currentYear; i >= currentYear-110;i-- ){
    years.push(i);
  }
  return years;
 }
populateYears(){
  if(this.state.years.length > 0){
    return this.state.years.map((year, index) =>(
      
      <option key={index} value={year}>{year}</option>
    ));

    
  }
}

populateMonths(){
  if(this.state.months.length > 0){
    return this.state.months.map((month, index)=>(
        <option key={month} value={index}>{month}</option>
    ));
  }
}

populateDays(){

if( (this.state.selectedYear != null || this.state.selectedYear != undefined) 
    && (this.state.selectedMonth != null || this.state.selectedMonth != undefined) ){
      
      let dayIterator = moment( this.state.selectedYear+"-"+(this.state.selectedMonth+1).toString(),"YYYY-M").daysInMonth();
      
      let days = [];
     for(let i =1; i <= dayIterator;i++){
       days.push(
         <option key={i} value={i}>{i}</option>
       );
     }
     return days;
    }
}
 calculateAge(){
   const yearbox = ReactDOM.findDOMNode(this.refs.yearSelect);
   const monthbox = ReactDOM.findDOMNode(this.refs.monthSelect);
   const daybox = ReactDOM.findDOMNode(this.refs.daySelect);
   let year = yearbox.options[yearbox.selectedIndex].value;
   let month = monthbox.options[monthbox.selectedIndex].value;
   let day = daybox.options[daybox.selectedIndex].value;

  
    console.log("Day value "+day);

 if( (year != null || year != '' ) &&
      (month != null || month != '' ) &&
      (day != null || day != '' )  ){
        const birthdate = new Date(year, month, day)
        return moment().diff(birthdate, 'years');
          
 }
  else return 0;
  }
sliderReader(event){
 ReactDOM.findDOMNode(this.refs.scoreOutput).value = ReactDOM.findDOMNode(this.refs.addtaskScore).value;

}

setIcon(event){
this.setState({selectedIndex:event.target.value});
}

setColorIndex(event){
this.setState({selectedColorIndex:event.target.value});
}

addTaskForm(event){
  this.state.adding ? this.setState({adding:false, selectedIndex:null,selectedColorIndex:null,selectedYear:null,selectedMonth:null,selectedDay:null, age: 0}) : this.setState({adding:true});  
}
renderProfiles() {
   
    return this.props.profiles.map((profile) => (   
     <Profile key={profile._id} profile={profile} icons={this.props.icons}/>
    ));
  }
 
renderIcons(){
   return this.props.icons.map((icon) => (
     <label key={"icon-"+icon.iconIndex} >
    <input type="radio" name="task-icon" ref={"icon-"+icon.iconIndex} value={icon.iconIndex} defaultChecked={icon.iconIndex === 0 ? "checked" : null} />
    <div  className="img-container" >
      <img src={icon.path} className="img-icon" />
    </div>
    </label>
     
     ));
}
renderColors(){
  let colors = ["#01AB9E","#E04D83","#F8D52C", "#F7A511", "#94C123", "#D73F24 "];
  let result =[];
  for(let i=0;i<colors.length;i++){
      result.push(
        <label key={"color-"+i }>
    <input type="radio" name="profile-color" ref={"color-"+i} value={i} defaultChecked={i === 0 ? "checked" : null} />
    <div  className="img-container" style={{backgroundColor: colors[i] }} >
      
    </div>
    </label>
      );
  }
  
   return result;
}
  render() {
    
    return (
      <div className="list-container">
        
        
        <ul>
          {this.renderProfiles()}
        </ul>

        <div className={"add-task-container" + (this.state.adding ? " adding-profile":'')}>
          <div id="addTaskRow" onClick={this.addTaskForm.bind(this)} >
          <div  className="img-container-blank" >
                <img src="icons/Avatar_blank.png" className="img-icon" />   
                       
            </div>
           <div className="add-profile-heading">Legg til familiemedlem</div>  
          </div>
          {this.state.adding &&
          
          <form id="add-task-form" onSubmit={this.handleSubmit.bind(this)}>
              <label htmlFor="addtaskTitle">Navn</label>
        <input type="text" 
               name="addtaskTitle"               
               maxLength="25"
               ref="addtaskTitle"
               className="add-task-text"
               required />
               
         <label htmlFor="yearSelect">Fødelsdag</label>
         <div className="center">
             <select name="yearSelect" ref="yearSelect" id="add-profile-year" onChange={this.yearSelected.bind(this)} required>
               <option value="" disabled selected>år</option>
              {this.populateYears()}
  
            </select>
            
             <select name="monthSelect" ref="monthSelect" id="add-profile-month" onChange={this.monthSelected.bind(this)} required>
               <option value="" disabled selected>måned</option>
              {this.populateMonths()}
  
            </select>
            <select name="daySelect" ref="daySelect" id="add-profile-day" onChange={this.daySelected.bind(this)}required>
               <option value="" disabled selected>dag</option>
              {this.populateDays()}
  
            </select>
            <div ref="ageLine"></div>
              { "Det betyr at du er "+this.state.age+" år"}
            </div>
        <label htmlFor="addtaskIcon">Bilde</label>
        <div className="icon-container" onChange={this.setIcon.bind(this)}>
            {this.renderIcons()}
        </div>
        <label htmlFor="addColor">Farge</label>
        <div className="color-container" onChange={this.setColorIndex.bind(this)}>
            {this.renderColors()}
        </div>
        
       
      <div className="center" >
         
              <input type="submit" value="Submit" className="submit-btn"/>
      </div>
       </form>
          }
      
       </div>
      </div>
    );
  }
}

export default createContainer(() => {
  Meteor.subscribe('profiles');
  Meteor.subscribe('icons');
  
  return {
    profiles: Profiles.find({}).fetch(),
    icons: Icons.find({type:"profile"}).fetch()
   
  };
}, ProfileList);