import { Meteor } from 'meteor/meteor';
import { Template } from 'meteor/templating';
import { ReactiveDict } from 'meteor/reactive-dict';

import { Groups } from '../models/group.js';
import { Profiles } from '../models/profile.js';
import { Tasks } from '../models/task.js';
import { Goals } from '../models/goal.js';

import './group.js';
import './task.js';
import './profile.js';
import './goal.js';
import './body.html';

Template.body.onCreated(function bodyOnCreated() {
  this.state = new ReactiveDict();
  Meteor.subscribe('groups');
  Meteor.subscribe('profiles');
  Meteor.subscribe('tasks');
  Meteor.subscribe('taskInstances');
  Meteor.subscribe('goals');
  Meteor.subscribe('goalInstances');
});

Template.body.helpers({
  groups() {
    return Groups.find();
  },
  profiles() {
    return Profiles.find();
  },
  tasks() {
    return Tasks.find({ isAssigned : { $eq : false }});
  },
  assignedTasks() {
    return Tasks.find({ isAssigned : { $eq : true }});
  },
  goals() {
    return Goals.find({ isAssigned : {$eq: false}});
  },
  assignedGoals() {
    return Goals.find({ isAssigned : { $eq : true }});
  }
});

Template.body.events({
  'submit .new-task'(event) {
    // Prevent default browser form submit
    event.preventDefault();
 
    // Get value from form element
    const target = event.target;
    const task   = {};
    task.name = target.name.value;
    task.isGroupTask = target.isGroupTask.value === "1";    
    task.score = target.score.value;
    task.iconIndex = target.iconIndex.value;
    task.age = target.age.value;
    task.personIndex = target.personIndex.value;
 
    // Insert a task into the collection
    Meteor.call('tasks.insert', task);

    // Clear form
    target.text.value = '';
  },
  'submit .new-goal'(event) {
    // Prevent default browser form submit
    event.preventDefault();
 
    // Get value from form element
    const target = event.target;
    const goal   = {};
    goal.name = target.name.value;
    goal.isGroup = target.isGroup.value === "1";    
    goal.score = target.score.value;
    goal.iconIndex = target.iconIndex.value;
    goal.age = target.age.value;
    goal.personIndex = target.personIndex.value;
 
    // Insert a task into the collection
    Meteor.call('goals.insert', goal);

    // Clear form
    target.text.value = '';
  },
  'submit .new-profile'(event) {
    // Prevent default browser form submit
    event.preventDefault();
 
    // Get value from form element
    const target  = event.target;
    const profile = {};
    const groupId = target.groupId.value;
    
    profile.name       = target.name.value;
    profile.birthDate  = target.birthDate.value;
    profile.colorIndex = target.colorIndex.value;
    profile.iconIndex  = target.iconIndex.value;
 
    // Insert a profile
    Meteor.call('profile.insert', groupId, profile);

    // Clear form
    target.name.value = '';
  },
  'submit .new-group'(event){
    // Prevent default browser form submit
    event.preventDefault();
 
    // Get value from form element
    const target    = event.target;
    const groupName = target.name.value;
    
    // Insert a profile
    Meteor.call('groups.insert', groupName);

    // Clear form
    target.name.value = '';
  },
  'submit .new-danger'(event){
    // Prevent default browser form submit
    event.preventDefault();
 
    // Get value from form element
    const target    = event.target;
    const groupName = target.name.value;
    
    // Insert a profile
    Meteor.call('app.reset', groupName);
  }
}

);
