﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using AssetBundles;
using System;

public class Assets : MonoBehaviour
{

    public static List<Sprite> TaskIcons { get; private set; }
    public static List<Sprite> ProfileIcons { get; private set; }

    public string taskIconAssetBundle = "taskicons";
    public string profileIconAssetBundle = "profileicons";

    private static Assets instance = null;

    // Use this for initialization
    IEnumerator Start()
    {
        if (instance == null)
            instance = this;
        else if (instance != this)
            Destroy(gameObject);
        yield return StartCoroutine(Initialize());

        // Load task icons
        yield return StartCoroutine(LoadTaskIcons());

        // Load profile icons
        yield return StartCoroutine(LoadProfileIcons());
    }



    // Initialize the downloading url and AssetBundleManifest object.
    protected IEnumerator Initialize()
    {
        // Don't destroy this gameObject as we depend on it to run the loading script.
        DontDestroyOnLoad(gameObject);

        // With this code, when in-editor or using a development builds: Always use the AssetBundle Server
        // (This is very dependent on the production workflow of the project. 
        // 	Another approach would be to make this configurable in the standalone player.)
#if DEVELOPMENT_BUILD || UNITY_EDITOR
        AssetBundleManager.SetDevelopmentAssetBundleServer();
#else
        // Use the following code if AssetBundles are embedded in the project for example via StreamingAssets folder etc:
        AssetBundleManager.SetSourceAssetBundleURL(Application.dataPath + "/");
        // Or customize the URL based on your deployment or configuration
        //AssetBundleManager.SetSourceAssetBundleURL("http://www.MyWebsite/MyAssetBundles");
#endif

        // Initialize AssetBundleManifest which loads the AssetBundleManifest object.
        var request = AssetBundleManager.Initialize();
        if (request != null)
            yield return StartCoroutine(request);
    }

    private IEnumerator LoadTaskIcons()
    {
        TaskIcons = new List<Sprite>();
        List<string> taskNames = new List<string>();
        for (int taskNumber = 15; taskNumber < 28; taskNumber++)
        {
            if (taskNumber == 21) continue;
            taskNames.Add("Assets_Prototype1_TaskIcons-" + taskNumber);
        }
        yield return StartCoroutine(LoadSpriteAssetBundle(taskIconAssetBundle, taskNames, TaskIcons));
    }

    private IEnumerator LoadProfileIcons()
    {
        ProfileIcons = new List<Sprite>();
        List<string> avatarNames = new List<string>();
        for (int avatarNumber = 2; avatarNumber < 12; avatarNumber += 2)
            avatarNames.Add("Assets_Prototype1_ProfileIcons-" + avatarNumber.ToString("D2"));
        for (int avatarNumber = 13; avatarNumber < 23; avatarNumber++)
            avatarNames.Add("Assets_Prototype1_ProfileIcons-" + avatarNumber);
        yield return StartCoroutine(LoadSpriteAssetBundle(taskIconAssetBundle, avatarNames, ProfileIcons));
    }

    private IEnumerator LoadSpriteAssetBundle(string assetbundle, List<string> assetNames, List<Sprite> sprites)
    {
        foreach (var assetName in assetNames)
        {
            var request = AssetBundleManager.LoadAssetAsync(profileIconAssetBundle, assetName, typeof(Sprite));
            if (request == null) continue;
            yield return StartCoroutine(request);
            TaskIcons.Add(request.GetAsset<Sprite>());
        }
    }
}
