﻿using UnityEngine;

[RequireComponent(typeof(Camera))]
public class CameraController : MonoBehaviour
{
    public new Camera camera;
    public Canvas canvas;
    public Canvas topCanvas;
    public Canvas bottomCanvas;

    private Vector3 initialPosition;
    private Vector3 finalPosition;
    private bool isMoving;
    private float dt;

    public void Start()
    {
        var height = (camera.orthographicSize * 2) / 0.015f;
        var width = (camera.orthographicSize * camera.aspect * 2) / 0.015f;
        ((RectTransform)canvas.transform).sizeDelta = new Vector2(width, height);
        ((RectTransform)topCanvas.transform).sizeDelta = new Vector2(width, height);
        ((RectTransform)bottomCanvas.transform).sizeDelta = new Vector2(width, height);
    }

    public void Update()
    {
        if (Input.GetKey(KeyCode.DownArrow))
            ScrollDown();
        else if (Input.GetKey(KeyCode.UpArrow))
            ScrollUp();
        if (isMoving)
        {
            dt += Time.deltaTime * Constants.Camera.cameraSpeed;
            Vector3 position = Vector3.Lerp(initialPosition, finalPosition, dt);
            camera.transform.position = position;
            canvas.transform.position = (Vector2)position;
            if (dt == 1)
                isMoving = false;
        }
    }

    public void ScrollDown()
    {
        if (camera.transform.position == Constants.Camera.upPosition)
            StartMovement(Constants.Camera.upPosition, Constants.Camera.downPosition);
    }

    public void ScrollUp()
    {
        if (camera.transform.position == Constants.Camera.downPosition)
            StartMovement(Constants.Camera.downPosition, Constants.Camera.upPosition);
    }

    private void StartMovement(Vector3 initialPosition, Vector3 finalPosition)
    {
        isMoving = true;
        dt = 0;
        this.initialPosition = initialPosition;
        this.finalPosition = finalPosition;
    }
}
