﻿using UnityEngine;
using System;
using System.Collections.Generic;
using System.Globalization;
using Data;

public class GameLogic : MonoBehaviour
{
    public Family family;

    public List<Sprite> avatars;
    public List<Sprite> taskIcons;
    public List<Sprite> ageGroups;
    public List<Sprite> groupIcons;
    public List<Color> colors;
    public List<Sprite> goalSprites;

    // Singleton instance
    private static GameLogic instance = null;
    private static bool destroyed = false;

    public static GameLogic Instance
    {
        get
        {
            if (destroyed)
                return null;
            if (instance == null)
                instance = new GameObject("GameLogic").AddComponent<GameLogic>();
            return instance;
        }
    }

    public void Awake()
    {
        if (instance == null)
            instance = this;
        else if (instance != this)
            Destroy(gameObject);
        DontDestroyOnLoad(gameObject);
        Initialize();
    }

    public void OnDestroy()
    {
        destroyed = true;
    }

    private void Initialize()
    {
        InitializeAssets();
        InitializeColors();
        InitializeGoals();
        InitializeScreenOrientation();
        Input.simulateMouseWithTouches = true;
    }

    private void InitializeGoals()
    {
        goalSprites = new List<Sprite>();
        for (var i = 1; i < 17; i++)
        {
            var sprite = Resources.Load<Sprite>("v11/Reward Icons/new_icons/Asset" + i);
            goalSprites.Add(sprite);
        }
    }

    private void InitializeAssets()
    {
        InitializeTaskIcons();
        InitializeAvatars();
        InitializeAgeIcons();
        InitializeGroupIcons();
    }

    private void InitializeAgeIcons()
    {
        ageGroups = new List<Sprite>();
        for (var i = 16; i < 23; i++)
            ageGroups.Add(Resources.Load<Sprite>("v4/OtherIcons/Assets_Prototype1_OtherIcons-" + i));
    }

    private void InitializeGroupIcons()
    {
        groupIcons = new List<Sprite>();
        for (var i = 15; i > 13; i--)
            groupIcons.Add(Resources.Load<Sprite>("v4/OtherIcons/Assets_Prototype1_OtherIcons-" + i));
    }

    private static void InitializeScreenOrientation()
    {
        Screen.autorotateToPortrait = true;
        Screen.autorotateToLandscapeRight = true;
        Screen.autorotateToLandscapeLeft = true;
        Screen.orientation = ScreenOrientation.LandscapeLeft;
    }

    private void InitializeTaskIcons()
    {
        taskIcons = new List<Sprite>();
        for (var taskNumber = 0; taskNumber <= 16; taskNumber++)
        {
            taskIcons.Add(Resources.Load<Sprite>("v11/Task Icons/Task_" + taskNumber));
        }
    }

    private void InitializeAvatars()
    {
        avatars = new List<Sprite>();
        for (int avatarNumber = 0; avatarNumber < 16; avatarNumber++)
            avatars.Add(Resources.Load<Sprite>("v11/Profile Icons/Avatar_" + avatarNumber));
    }

    private void InitializeColors()
    {
        colors = Constants.Colors.profileColors.ConvertAll(HexToColor);
    }

    public static Color HexToColor(string hex)
    {
        hex = hex.Replace("0x", "");  //in case the string is formatted 0xFFFFFF
        hex = hex.Replace("#", "");   //in case the string is formatted #FFFFFF
        byte a = 255;                 //assume fully visible unless specified in hex
        byte r = byte.Parse(hex.Substring(0, 2), NumberStyles.HexNumber);
        byte g = byte.Parse(hex.Substring(2, 2), NumberStyles.HexNumber);
        byte b = byte.Parse(hex.Substring(4, 2), NumberStyles.HexNumber);
        //Only use alpha if the string has enough characters
        if (hex.Length == 8)
            a = byte.Parse(hex.Substring(4, 2), NumberStyles.HexNumber);
        var color32 = new Color32(r, g, b, a);
        return color32;
    }

    public static int AgeInYears(DateTime dateOfBirth)
    {
        var now = DateTime.Now;
        return (now.Year - dateOfBirth.Year - 1) +
            (((now.Month > dateOfBirth.Month) ||
            ((now.Month == dateOfBirth.Month) && (now.Day >= dateOfBirth.Day))) ? 1 : 0);
    }
}
