﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

/*
 * In charge of managing homework tasks
 */
public class HomeworkTaskManager : MonoBehaviour {

    public GameObject homeworkPrefab;
    public Sprite mathematicsHomework;
    public Sprite norwegianHomework;

    private bool isFound = false;
    private Transform spawnpoint;
    private Color colorBG;

    // Test data
    private bool haveSpawnedExtraTask = false;
    private HomeworkTask[] homework1 = { new HomeworkTask("Hvordan tror du man regner ut '312:12'? Skriv svaret ditt i arbeidsboken din.", false, new HomeworkLink[] { }), new HomeworkTask("Se YouTube-videoen om flersifret divisjon før du fortsetter til neste oppgave.", false, new HomeworkLink[] { new HomeworkLink("Flersifret divisor", "https://www.youtube.com/watch?v=gLRay1kvYP0") }), new HomeworkTask("Prøv den nye kunnskapen din ved å regne ut oppgave 1.2, 1.3 og 1.4 i matematikkboken din.", false, new HomeworkLink[] { }) };
    private HomeworkTask[] homework2 = { new HomeworkTask("Les side 112-116 i leseboken din.", false, new HomeworkLink[] { }), new HomeworkTask("Gjør oppgavene som hører til leseleksen på side 117.", false, new HomeworkLink[] {  }) };
    private HomeworkTask[] homeworkExtra = { new HomeworkTask("Gjør quizen på Quizlet.", false, new HomeworkLink[] { new HomeworkLink("Quizlet", "https://quizlet.com/138749958/write") }), new HomeworkTask("Gjør quizen på Quizlet.", false, new HomeworkLink[] { new HomeworkLink("Quizlet", "https://quizlet.com/272093295/write") }) };

    void Update () {
        //Find TaskRoot/TodoPool:3/Avatar
        if (isFound == false)
        {
            GameObject todoPool = FindRightMostUser();

            if (todoPool != null)
            {
                isFound = true;
                spawnpoint = todoPool.transform;
                colorBG = todoPool.GetComponentInChildren<SpriteRenderer>().color;

                SpawnHomework(homework1, Subject.MATHEMATICS);
                SpawnHomework(homework2, Subject.NORWEGIAN, new Vector3(1, 0, 0));
            }
        }
    }

    private GameObject FindRightMostUser()
    {
        int todoPoolIndex = ((GameObject.Find("PoolRoot").transform.childCount - 2) / 2) -1;

        return GameObject.Find("PoolRoot/TodoPool:" + todoPoolIndex + "/Avatar");
    }

    public void SpawnExtraTask()
    {
        if (!haveSpawnedExtraTask)
        {
            SpawnHomework(homeworkExtra, Subject.MATHEMATICS);
            haveSpawnedExtraTask = true;
        }
    }

    public void SpawnHomework(HomeworkTask[] homework, Subject subject)
    {
        var homeworkTask = Instantiate(homeworkPrefab);

        homeworkTask.transform.SetParent(GameObject.Find("TaskRoot").transform, false);
        homeworkTask.transform.position = spawnpoint.position;
        HomeworkTaskController homeworkTaskController = homeworkTask.GetComponent<HomeworkTaskController>();

        homeworkTaskController.SetBackgroundColor(colorBG);
        homeworkTaskController.SetTarget(GameObject.Find("TopCanvas/5C").transform);
        homeworkTaskController.SetHomework(homework);
        homeworkTaskController.SetIcon(GetSubjectIcon(subject));
    }

    public void SpawnHomework(HomeworkTask[] homework, Subject subject, Vector3 offset)
    {
        var homeworkTask = Instantiate(homeworkPrefab);

        homeworkTask.transform.SetParent(GameObject.Find("TaskRoot").transform, false);
        homeworkTask.transform.position = spawnpoint.position + offset;
        HomeworkTaskController homeworkTaskController = homeworkTask.GetComponent<HomeworkTaskController>();

        homeworkTaskController.SetBackgroundColor(colorBG);
        homeworkTaskController.SetTarget(GameObject.Find("TopCanvas/5C").transform);
        homeworkTaskController.SetHomework(homework);
        homeworkTaskController.SetIcon(GetSubjectIcon(subject));
    }

    private Sprite GetSubjectIcon(Subject subject)
    {
        if(subject == Subject.MATHEMATICS)
        {
            return mathematicsHomework;
        }
        else
        {
            return norwegianHomework;
        }
    }

}

