﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

/*
 * Controls particles the tasks dissolve into
 */
public class ParticleController : MonoBehaviour
{

    private Transform endTarget;
    private Vector3 randomOffset;
    [SerializeField]
    private float baseSpeed = 2;
    [SerializeField]
    private float speedModifier = 1.2f;
    private float speed = 2;
    private bool offsetReached = false;
    private bool finishedWaiting = false;

    private Vector3 velocity = new Vector3();

    void Start()
    {
        speed = baseSpeed;
    }

    void Update()
    {
        if (endTarget != null && randomOffset != null)
        {
            if ((transform.position - endTarget.position).sqrMagnitude < 1f)
            {
                Debug.Log("Target destroyed!");
                Destroy(gameObject);
            }
            else
            {
                if (!offsetReached)
                {

                    if ((transform.position - randomOffset).sqrMagnitude < 1f)
                    {
                        offsetReached = true;
                        StartCoroutine(DramaticPause());
                    }
                    else
                    {
                        transform.position = Vector3.SmoothDamp(transform.position, randomOffset, ref velocity, 0.4f); //new Vector3(Mathf.MoveTowards(transform.position.x, randomOffset.x, speed * Time.deltaTime), Mathf.MoveTowards(transform.position.y, randomOffset.y, speed * Time.deltaTime), transform.position.z);
                    }
                }
                else
                {
                    if (finishedWaiting)
                    {
                        transform.position = Vector3.MoveTowards(transform.position, endTarget.position, speed * Time.deltaTime);//new Vector3(Mathf.MoveTowards(transform.position.x, endTarget.position.x, speed * Time.deltaTime), Mathf.MoveTowards(transform.position.y, endTarget.position.y, speed * Time.deltaTime), transform.position.z);
                        speed *= speedModifier;
                    }
                }
            }
        }
    }

    IEnumerator DramaticPause()
    {
        yield return new WaitForSeconds(0.5f);
        finishedWaiting = true;
    }

    public void SetRandomOffset(Vector3 vector)
    {
        this.randomOffset = this.transform.position + vector;
    }

    public void SetEndTarget(Transform target)
    {
        this.endTarget = target;
    }
}
