﻿using System;
using System.Collections;
using System.Collections.Generic;
using Data;
using UnityEngine;
using UnityEngine.EventSystems;
using System.Linq;

public class GoalPool : MonoBehaviour, IPointerClickHandler
{
    public BoxCollider2D boxCollider;
    public List<GameObject> goalBalls = new List<GameObject>();
    public Queue<Goal> pendingGoals = new Queue<Goal>();
    public GameObject goalBallPrefab;

    public const int MaxGoals = 4; // TODO: fix this
    private bool _isSpawning;
    private bool _isMovingGoal;

    public void OnTriggerStay2D(Collider2D other)
    {
        var goalBall = other.GetComponent<GoalBall>();
        if (goalBall == null || !IsInsidePool(other))
        {
            return;
        }

        var circleCollider = other.GetComponent<CircleCollider2D>();
        if (circleCollider.isTrigger)
        {
            circleCollider.isTrigger = false;
            other.GetComponent<Rigidbody2D>().isKinematic = false;
            StartCoroutine(MoveBallToSide(other.gameObject));
        }
    }

    private bool IsInsidePool(Collider2D other)
    {
        return boxCollider.bounds.Contains(other.bounds.max) && boxCollider.bounds.Contains(other.bounds.min);
    }

    private IEnumerator MoveBallToSide(GameObject goalBall)
    {
        if (!goalBalls.Contains(goalBall))
        {
            goalBalls.Add(goalBall);
        }

        var isMoveLeft = true;
        if (goalBalls.Count > 1)
        {
            var prevBall = goalBalls[goalBalls.Count - 2].GetComponent<GoalBall>();
            var spaceNeeded = goalBall.GetComponent<GoalBall>().collider.bounds.extents.x + prevBall.collider.bounds.extents.x;
            isMoveLeft = prevBall.transform.position.x + spaceNeeded < transform.position.x;
        }

        var rigidBody = goalBall.GetComponent<Rigidbody2D>();
        while (!Mathf.Approximately(rigidBody.velocity.y, 0))
        {
            yield return new WaitForSeconds(0.2f);
        }
        _isMovingGoal = true;
        var dir = isMoveLeft ? -1 : 1;
        const float speed = 4f;
        rigidBody.velocity = new Vector2(dir * speed, 0);
        while (!rigidBody.isKinematic)
        {
            yield return null;
        }

        rigidBody.velocity = new Vector2(0, 0);
        rigidBody.angularVelocity = 0;
    }

    public void OnEnable()
    {
        if (goalBalls.Count <= 0)
        {
            return;
        }

        var goalBall = goalBalls[goalBalls.Count - 1];
        if (!goalBall.GetComponent<Rigidbody2D>().isKinematic)
        {
            StartCoroutine(SpawnPendingGoals(goalBall));
        }
    }

    public void OnCollisionStay2D(Collision2D collision)
    {
        if (Math.Abs(collision.rigidbody.velocity.x) < 1.5 && _isMovingGoal)
        {
            collision.rigidbody.isKinematic = true;
            _isMovingGoal = false;
        }
    }

    public void OnPointerClick(PointerEventData eventData)
    {
        var colliders = Physics2D.OverlapPointAll(Camera.main.ScreenToWorldPoint(Input.mousePosition));
        foreach (var collider in colliders)
        {
            var goalBall = collider.GetComponent<GoalBall>();
            if (goalBall != null)
            {
                goalBall.OnPointerClick(eventData);
            }
        }
    }

    public void AddGoalDirectly(Goal goal)
    {
        pendingGoals.Enqueue(goal);
        if (!_isSpawning)
        {
            StartCoroutine(SpawnPendingGoals());
        }
    }

    private IEnumerator SpawnPendingGoals(GameObject goalBall = null)
    {
        _isSpawning = true;
        while (goalBall != null || (pendingGoals.Count > 0 && HasRoomForMore()))
        {
            if (goalBall == null)
            {
                var goal = pendingGoals.Dequeue();
                goalBall = Instantiate(goalBallPrefab);
                goalBall.GetComponent<GoalBall>().Goal = goal;
                goalBall.transform.position = new Vector2(transform.position.x, transform.position.y - 0.9f); // TODO: fix hotfix
                goalBall.GetComponent<CircleCollider2D>().isTrigger = false;
                goalBall.GetComponent<Rigidbody2D>().isKinematic = false;
            }
            yield return StartCoroutine(MoveBallToSide(goalBall));
            goalBall = null;
        }
        _isSpawning = false;
    }

    public void RemoveGoal(Goal goal)
    {
        var goalGo = goalBalls.FirstOrDefault(g => g.GetComponent<GoalBall>().Goal == goal);
        if (goalGo != null)
            goalBalls.Remove(goalGo);
    }

    public bool HasRoomForMore()
    {
        return goalBalls.Count < MaxGoals;
    }

    public void Show()
    {
        foreach (var goalBall in goalBalls)
        {
            goalBall.SetActive(true);
        }
        gameObject.SetActive(true);
    }

    public void Hide()
    {
        gameObject.SetActive(false);
        foreach (var goalBall in goalBalls)
        {
            goalBall.SetActive(false);
        }
    }
}
