﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.EventSystems;
using System;
using System.Linq;
using Data;
using Manage.Services.Synchronizers;

public class RewardPool : MonoBehaviour, IPointerClickHandler
{
    public bool isFamilyPool;
    public SpriteRenderer poolLevelBottom;
    public SpriteRenderer poolLevelTop;
    public SpriteRenderer poolBg;
    public BoxCollider2D poolContainer;
    public GameObject goalMenuPrefab;
    public GameObject goalBarPrefab;
    public GameObject goalBallPrefab;
    public GameObject checkmark;
    public Sprite goalBackground;
    public GameObject goalPoolPrefab;
    public bool isHidden;
    public Sprite plusSprite;

    private Queue<GameObject> pendingTasks;
    private bool isSpawning;
    private bool isCompletingGoal;

    public PersonSynchronizer PersonSynchronizer { get; private set; }
    public GoalSynchronizer GoalSynchronizer { get; private set; }
    public TaskSynchronizer TaskSynchronizer { get; private set; }

    public float PoolMaxScore { get; set; }
    private float poolMaxLevel;
    private float poolLevelScaleFactor;
    private float poolLevel;

    private Canvas _canvas;
    private Dictionary<GameObject, Vector2> taskPositions;
    private Person person;
    private Queue<Goal> completedGoals;
    private GameObject goalBar;
    private Transform goalButtonObject;
    private float defaultGoalBarY;
    private GameObject goalPool;

    public GoalPool GoalPool
    {
        get
        {
            return goalPool.GetComponent<GoalPool>();
        }
    }

    public Goal Goal { get; private set; }

    public Person Person
    {
        get { return person; }
        set
        {
            person = value;
            if (person != null)
            {
                var color = person.Color;
                color.a = 0.2f;
                poolLevelTop.color = color;
                poolLevelBottom.color = color;
            }
        }
    }

    #region UnityFunctions
    public void Awake()
    {
        PoolMaxScore = 0;
        if (isFamilyPool)
        {
            var spriteWidth = poolBg.bounds.extents.x;
            var screenWidth = Camera.main.orthographicSize * Camera.main.aspect;
            transform.localScale = new Vector2((screenWidth / spriteWidth) * 0.95f, transform.localScale.y); // TODO fix dirty constant.
            PoolMaxScore = 400;
            poolLevelScaleFactor = 0.8f;
        }
        var bounds = poolBg.bounds;
        poolMaxLevel = bounds.center.y * 0.4f + bounds.max.y * 0.6f;
        if (poolLevelScaleFactor == 0)
            poolLevelScaleFactor = 0.845f; // TODO: calculate factor

        pendingTasks = new Queue<GameObject>();
        poolLevelTop.transform.localScale = new Vector2(1, 0);

        taskPositions = new Dictionary<GameObject, Vector2>();
        if (_canvas == null)
            _canvas = GameObject.Find("BottomCanvas").GetComponent<Canvas>();
        var serverObject = GameObject.Find("Server");
        PersonSynchronizer = serverObject.GetComponent<PersonSynchronizer>();
        GoalSynchronizer = serverObject.GetComponent<GoalSynchronizer>();
        TaskSynchronizer = serverObject.GetComponent<TaskSynchronizer>();
        completedGoals = new Queue<Goal>();
    }

    public void Start()
    {
        goalBar = Instantiate(goalBarPrefab);
        goalBar.transform.SetParent(_canvas.transform, false);
        var bounds = poolBg.bounds;
        var width = bounds.size.x * 0.995f;
        var rectTransform = (RectTransform)goalBar.transform;
        var canvasScale = ((RectTransform)_canvas.transform).localScale.x;
        rectTransform.sizeDelta = new Vector2(width / canvasScale, rectTransform.sizeDelta.y);
        defaultGoalBarY = rectTransform.position.y;
        rectTransform.position = new Vector2(transform.position.x, defaultGoalBarY);
        goalButtonObject = goalBar.transform.Find("Button");
        goalButtonObject.GetComponent<Button>().onClick.AddListener(OnGoalButtonPress);
        InitGoalPool();

        if (!isFamilyPool)
            PoolMaxScore = 400f / PersonSynchronizer.Persons.Count;
        SetGoalBarGoal();
    }

    public void Update()
    {
        // TODO: Maybe get this out of update and into a Coroutine
        var yScale = poolLevelTop.transform.localScale.y;
        var targetScale = (poolLevel / PoolMaxScore) * poolLevelScaleFactor;
        const float poolLevelSpeed = 0.01f;
        if (Goal != null && Mathf.Abs(targetScale - yScale) > poolLevelSpeed)
        {
            yScale += poolLevelSpeed * Mathf.Sign(targetScale - yScale);
            poolLevelTop.transform.localScale = new Vector2(1, yScale);
        }

        if (poolLevelTop.gameObject.activeSelf
            && Goal != null
            && Goal.IsCompleted.HasValue
            && Goal.IsCompleted.Value
            && Math.Abs(yScale) <= poolLevelSpeed)
        {
            GoalAnimationFinished();
        }
    }

    public void OnDisable()
    {
        isSpawning = false;
    }

    public void OnTriggerExit2D(Collider2D collision)
    {
        var taskController = collision.GetComponent<TaskBallController>();
        if (taskController != null)
            taskController.Task.Person = null;
    }
    #endregion UnityFunctions

    public void AddGoal(Goal goal)
    {
        if (!goal.IsCompleted.HasValue || !goal.IsCompleted.Value)
        {
            Goal = goal;
            InitGoal();
        }
        else if (goal.IsCompleted.Value)
        {
            if (goalPool != null)
                goalPool.GetComponent<GoalPool>().AddGoalDirectly(goal);
            else
                completedGoals.Enqueue(goal);
        }
    }

    private void InitGoal()
    {
        if (goalButtonObject != null)
        {
            SetGoalButtonSprite(Goal.Sprite);
        }

        if (poolLevel > 0)
        {
            poolLevelBottom.gameObject.SetActive(true);
            poolLevelTop.gameObject.SetActive(true);
        }

        if (!Goal.Score.HasValue)
        {
            Goal.Score = 100;
        }
        SetGoalBarGoal();
    }

    private void SetGoalButtonSprite(Sprite goalIcon)
    {
        var goalIconObject = goalButtonObject.FindChild("GoalIcon");
        var goalBackgroundIcon = goalButtonObject.GetComponent<Image>();
        if (goalIcon != null)
        {
            goalIconObject.gameObject.SetActive(true);
            goalBackgroundIcon.sprite = goalBackground;
            goalBackgroundIcon.color = Constants.Colors.goalBackground;
            goalIconObject.GetComponent<Image>().sprite = goalIcon;
        }
        else
        {
            goalIconObject.gameObject.SetActive(false);
            goalBackgroundIcon.sprite = plusSprite;
            goalBackgroundIcon.color = Color.white;
        }
    }

    private void SetGoalBarGoal()
    {
        if (goalBar != null && Goal != null)
        {
            if (Goal.Score > PoolMaxScore)
                Goal.Score = (int)PoolMaxScore;
            float yPosition = poolLevelTop.transform.position.y;
            yPosition += (Goal.Score.Value / PoolMaxScore) * (poolMaxLevel - yPosition);
            goalBar.transform.position = new Vector2(goalBar.transform.position.x, yPosition);
            SetGoalButtonSprite(Goal.Sprite);
        }
    }

    private void InitGoalPool() // TODO kjellre 2017-03-11 Why is this not in the PoolController?!
    {
        if (goalPoolPrefab == null)
        {
            return;
        }

        var poolRoot = GameObject.Find("PoolRoot");
        goalPool = Instantiate(goalPoolPrefab);
        goalPool.transform.parent = poolRoot == null ? null : poolRoot.transform;
        var owner = person == null ? "Family" : person.Name;
        goalPool.name = "GoalPool:" + owner;

        var yPosition = poolBg.bounds.min.y - 0.32f; // TODO: fix hard code
        var goalPoolScript = goalPool.GetComponent<GoalPool>();
        float poolScaleX = poolBg.bounds.size.x / goalPool.GetComponent<SpriteRenderer>().bounds.size.x;
        goalPool.transform.localScale = new Vector2(poolScaleX, transform.localScale.y);
        goalPool.transform.position = new Vector2(transform.position.x, yPosition);
        goalPoolScript.goalBallPrefab = goalBallPrefab;

        while (completedGoals.Count > 0)
        {
            var goal = completedGoals.Dequeue();
            goalPoolScript.AddGoalDirectly(goal);
        }
    }

    public void AssignGoal(Goal goal)
    {
        Goal = goal;
        UpdatePoolLevel(0);
        SetGoalButtonSprite(goal.Sprite);
        GoalSynchronizer.AssignGoal(Person, goal);
    }

    public void UpdatePoolLevel(float levelIncrease)
    {
        if (poolLevel == 0 && levelIncrease > 0 && Goal != null)
        {
            poolLevelBottom.gameObject.SetActive(true);
            poolLevelTop.gameObject.SetActive(true);
        }
        poolLevel += levelIncrease;
        poolLevel = Mathf.Clamp(poolLevel, 0, PoolMaxScore);
        if (Goal != null && !Goal.IsCompleted.Value && poolLevel >= Goal.Score)
            CompleteGoal();
    }

    private void CompleteGoal()
    {
        Goal.IsCompleted = true;
        poolLevel = Goal.Score.Value;
        goalButtonObject.localScale = new Vector2(1.25f, 1.25f);
        goalButtonObject.GetComponent<Image>().color = Constants.Colors.goalCompleted;
    }

    private void GoalAnimationFinished()
    {
        poolLevelTop.gameObject.SetActive(false);
        poolLevelBottom.gameObject.SetActive(false);
        StartCoroutine(MoveGoalToGoalPool());
        goalBar.transform.position = new Vector2(goalBar.transform.position.x, defaultGoalBarY);
        goalButtonObject.transform.localScale = new Vector2(1f, 1f);
        SetGoalButtonSprite(null);
    }

    private IEnumerator MoveGoalToGoalPool()
    {
        var goalBall = Instantiate(goalBallPrefab);
        goalBall.GetComponent<GoalBall>().Goal = Goal;
        goalBall.transform.position = goalButtonObject.transform.position;
        Goal = null;
        yield return null;
    }

    public void OnGoalButtonPress()
    {
        if (Goal != null && Goal.IsCompleted.HasValue && Goal.IsCompleted.Value)
        {
            if (goalPool != null && !goalPool.GetComponent<GoalPool>().HasRoomForMore())
                return;

            var tasks = taskPositions.Keys
                .Select(g => g.GetComponent<TaskBallController>())
                .ToList();
            foreach (var t in tasks)
            {
                t.Task.IsGoaled = true;
            }
            GoalSynchronizer.CompleteGoal(Goal, tasks);
            StartCoroutine(EatTasks());
            isCompletingGoal = true;
        }
        else
        {
            var goalMenu = Instantiate(goalMenuPrefab);
            goalMenu.GetComponent<GoalMenu>().rewardPool = this;
            goalMenu.transform.SetParent(_canvas.transform, false);
            var point = (RectTransform)goalMenu.transform.Find("Background").Find("Point");
            point.position = new Vector2(transform.position.x, point.position.y);
        }
    }

    private IEnumerator EatTasks()
    {
        var rand = new System.Random();
        while (taskPositions.Count > 0)
        {
            var task = taskPositions.ElementAt(rand.Next(taskPositions.Count)).Key;
            StartCoroutine(EatTask(task));
            yield return new WaitForSeconds(0.03f);
        }
        yield break;
    }

    private IEnumerator EatTask(GameObject task)
    {
        taskPositions.Remove(task);
        var taskController = task.GetComponent<TaskBallController>();
        UpdatePoolLevel(-taskController.Task.Score);
        taskController.DisablePhysics();
        float speed = 0.01f;
        float acc = 0.007f;
        while (Vector2.Distance(task.transform.position, goalButtonObject.transform.position) > speed)
        {
            var dir = (goalButtonObject.transform.position - task.transform.position).normalized;
            dir.Scale(new Vector2(speed, speed));
            task.transform.position += dir;
            speed += acc;
            yield return null;
        }
        TaskSynchronizer.RemoveTask(task);
        Destroy(task);
    }

    public void RemoveTask(GameObject taskGo)
    {
        if (taskPositions.Remove(taskGo))
            UpdatePoolLevel(-taskGo.GetComponent<TaskBallController>().Task.Score);
    }

    public void AddTask(GameObject go)
    {
        var taskController = go.GetComponent<TaskBallController>();
        if (!IsInPool(taskController.collider))
        {
            pendingTasks.Enqueue(go);
            if (!isSpawning && !isHidden)
                SpawnPendingTasks();
        }
        else
        {
            UpdatePoolLevel(go.GetComponent<TaskBallController>().Task.Score);
            if (!taskPositions.Keys.Contains(go))
                taskPositions.Add(go, taskController.transform.position);
        }
    }

    private void SpawnPendingTasks()
    {
        if (taskPositions.Count == 0)
            StartCoroutine(SpawnTasksInPool());
        else
            StartCoroutine(SpawnTasksInPoolOld());
    }

    private IEnumerator SpawnTasksInPoolOld()
    {
        isSpawning = true;
        var random = new System.Random();
        while (pendingTasks.Count != 0)
        {
            var task = pendingTasks.Dequeue().GetComponent<TaskBallController>();
            var yPosition = transform.position.y + poolBg.bounds.extents.y;
            task.Task.IsUpdateEnabled = false;
            task.Task.HasSettled = true;
            task.transform.position = new Vector2(transform.position.x, yPosition);
            task.EnablePhysics();
            task.rigidBody.velocity = new Vector2(((float)random.NextDouble() - 0.5f) * 2f, -1f);

            yield return new WaitForSeconds(0.7f);

            foreach (var renderer in task.GetComponentsInChildren<SpriteRenderer>())
            {
                renderer.enabled = true;
            }
            task.Task.IsUpdateEnabled = true;
            taskPositions.Add(task.gameObject, task.transform.position);
            float levelIncrease = task.Task.Score; // TODO: fix this quick fix
            if (task.Task.IsApproved)
                levelIncrease *= 1.4f;
            UpdatePoolLevel(levelIncrease);
        }
        isSpawning = false;
    }

    private IEnumerator SpawnTasksInPool()
    {
        isSpawning = true;
        var x = poolContainer.bounds.min.x;
        var y = poolContainer.bounds.min.y;
        float nextY = y;
        while (pendingTasks.Count != 0)
        {
            var taskGo = pendingTasks.Dequeue();
            if (taskGo == null) continue;
            var taskController = taskGo.GetComponent<TaskBallController>();
            taskController.Task.IsUpdateEnabled = false;
            taskController.Task.HasSettled = true;

            float radius = taskController.GetComponent<CircleCollider2D>().bounds.extents.x;
            float diameter = radius * 2;
            if (y + diameter * 1.2f > poolContainer.bounds.max.y)
                break;
            if (x + diameter > poolContainer.bounds.max.x)
            {
                y = nextY;
                x = poolContainer.bounds.min.x;
            }
            taskController.transform.position = new Vector2(x + radius, y + radius);
            taskController.GetComponent<TaskBallController>().EnablePhysics();
            x += diameter;
            nextY = Mathf.Max(nextY, y + diameter);

            taskController.EnablePhysics();
            foreach (var renderer in taskController.GetComponentsInChildren<SpriteRenderer>())
                renderer.enabled = true;
            yield return new WaitForSeconds(0.02f);
            taskController.Task.IsUpdateEnabled = true;
            taskPositions.Add(taskController.gameObject, taskController.transform.position);
            float levelIncrease = taskController.Task.Score; // TODO: fix this quick fix
            if (taskController.Task.IsApproved)
                levelIncrease *= 1.4f;
            UpdatePoolLevel(levelIncrease);
        }
        isSpawning = false;
        if (pendingTasks.Count > 0)
            StartCoroutine(SpawnTasksInPoolOld());
    }

    public void Hide()
    {
        isHidden = true;
        gameObject.SetActive(false);
        var keys = new List<GameObject>(taskPositions.Keys);
        foreach (var task in keys)
        {
            taskPositions[task] = task.transform.position;
            task.GetComponent<TaskBallController>().DisablePhysics();
            var renderers = task.GetComponentsInChildren<SpriteRenderer>();
            foreach (var renderer in renderers)
            {
                renderer.enabled = false;
            }
        }
        if (goalBar != null)
            goalBar.SetActive(false);
        if (checkmark != null)
            checkmark.SetActive(false);
        if (goalPool != null)
            goalPool.GetComponent<GoalPool>().Hide();
    }

    public void Show()
    {
        isHidden = false;
        gameObject.SetActive(true);
        var keys = new List<GameObject>(taskPositions.Keys);
        if (keys.Count == 0)
        {
            keys = pendingTasks.ToList();
        }
        foreach (var task in keys)
        {
            if (taskPositions.ContainsKey(task))
                task.transform.position = taskPositions[task];
            if (!pendingTasks.Contains(task))
            {
                task.GetComponent<TaskBallController>().EnablePhysics();
                foreach (var renderer in task.GetComponentsInChildren<SpriteRenderer>())
                    renderer.enabled = true;
            }
        }
        if (!isSpawning)
            SpawnPendingTasks();
        if (goalBar != null)
            goalBar.SetActive(true);
        if (checkmark != null)
            checkmark.SetActive(true);
        if (goalPool != null)
            goalPool.GetComponent<GoalPool>().Show();

    }

    public void ToggleShowHide()
    {
        if (isHidden)
        {
            Show();
        }
        else
        {
            Hide();
        }
    }

    public void OnPointerClick(PointerEventData eventData)
    {
        var colliders = Physics2D.OverlapPointAll(Camera.main.ScreenToWorldPoint(Input.mousePosition));
        foreach (var collider in colliders)
        {
            var task = collider.GetComponent<TaskBallController>();
            if (task != null)
            {
                task.Select();
            }
        }
    }

    public bool IsInPool(Collider2D other)
    {
        return poolContainer.bounds.Contains(other.bounds.max) &&
               poolContainer.bounds.Contains(other.bounds.min);
    }
}
