﻿using System;
using System.ComponentModel;
using Constants;
using Data;
using UnityEngine;

public class TaskView : MonoBehaviour
{
    private Task task;
    private Person person;

    public SpriteRenderer backgroundSprite;
    public SpriteRenderer taskSprite;
    public SpriteRenderer ringSprite;

    public Task Task
    {
        get { return task; }
        set
        {
            if (task != null)
                task.PropertyChanged -= TaskPropertyChanged;
            task = value;
            task.PropertyChanged += TaskPropertyChanged;
            taskSprite.sprite = task.TaskIcon;
            if (task.Person != null)
            {
                person = task.Person;
                person.PropertyChanged += PersonPropertyChanged;
                backgroundSprite.color = person.Color;
                SetPhotoTaskColors(person);
            }
        }
    }

    // This function is called when the MonoBehaviour will be destroyed
    public void OnDestroy()
    {
        if (person != null)
            person.PropertyChanged -= PersonPropertyChanged;
        if (task != null)
            task.PropertyChanged -= TaskPropertyChanged;
    }

    private void PersonPropertyChanged(object sender, PropertyChangedEventArgs e)
    {
        if (e.PropertyName == PersonProperties.Color)
        {
            backgroundSprite.color = person.Color;
            SetPhotoTaskColors(person);
        }
    }

    private void SetPhotoTaskColors(Person person)
    {
        if (ringSprite != null)
        {
            if (person != null)
            {
                float h, s, v;
                Color.RGBToHSV(person.Color, out h, out s, out v);
                v -= 30f / 255f; // TODO fix hard code?
                ringSprite.color = Color.HSVToRGB(h, s, v);
                var taskColor = taskSprite.color;
                taskColor.a = 0.75f;
                taskSprite.color = taskColor;
            }
            else
                ringSprite.color = GameLogic.HexToColor("#402D39"); // TODO: fix hard code
        }
    }

    private void TaskPropertyChanged(object sender, PropertyChangedEventArgs e)
    {
        switch (e.PropertyName)
        {
            case TaskProperties.Person:
                if (person != null)
                    person.PropertyChanged -= PersonPropertyChanged;
                person = task.Person;
                if (person != null)
                {
                    backgroundSprite.color = task.Person.Color;
                    person.PropertyChanged += PersonPropertyChanged;
                }
                else
                    backgroundSprite.color = Constants.Colors.undefinedBackgroundColor;
                SetPhotoTaskColors(person);
                break;
            case TaskProperties.TaskIcon:
                taskSprite.sprite = task.TaskIcon;
                break;
            default: // Do nothing
                break;
        }
    }

    public void SetSortingLayer(string sortingLayer)
    {
        backgroundSprite.sortingLayerName = sortingLayer;
        taskSprite.sortingLayerName = sortingLayer;
        if (ringSprite != null)
            ringSprite.sortingLayerName = sortingLayer;
    }
}
