﻿using UnityEngine;
using UnityEngine.UI;
using UnityEngine.SceneManagement;
using System.Collections;
using System.Collections.Generic;
using Data;
using Manage.Services.Synchronizers;
using Orchestration;

public class FamilyEditMenu : MonoBehaviour, IStartupComponent
{
    public RectTransform canvas;
    public Text familyText;
    public Button doneButton;
    public Button addMemberButton;
    public RectTransform avatarScrollView;
    public RectTransform scrollContent;
    public PersonSynchronizer personSynchronizer;

    public static float avatarWidth = 300;
    public static float avatarSpace = 20;

    private float maxScrollViewVidth;
    private float scrollViewWidth;
    private float addButtonWidth;
    private List<Person> persons;

    #region IStartupComponent

    public int InitOrder { get { return 10; } }
    public bool IsInitComplete { get; private set; }

    public void Initialize()
    {
        persons = personSynchronizer.Persons;
        familyText.text = personSynchronizer.Family.Name;
        InitializeScrollView();
        InitializeAvatars();

        IsInitComplete = true;
    }

    #endregion

    private void InitializeScrollView()
    {
        int members = persons.Count;
        if (members == 0)
        {
            avatarScrollView.FindChild("Scroll View").gameObject.SetActive(false);
            doneButton.enabled = false;
        }
        addButtonWidth = (addMemberButton == null) ? 0 : ((RectTransform)addMemberButton.transform).sizeDelta.x;
        maxScrollViewVidth = canvas.sizeDelta.x;
        scrollViewWidth = members * avatarWidth + (members - 1) * avatarSpace + addButtonWidth;
        avatarScrollView.sizeDelta = new Vector2(Mathf.Min(scrollViewWidth, maxScrollViewVidth),
            avatarScrollView.sizeDelta.y);
        if (scrollViewWidth > maxScrollViewVidth)
            scrollContent.offsetMax = new Vector2(scrollViewWidth - maxScrollViewVidth, scrollContent.offsetMax.y);
    }

    private void InitializeAvatars()
    {
        for (var i = 0; i < persons.Count; i++)
        {
            var person = persons[i];
            var personUi = Instantiate(Resources.Load<RectTransform>("Prefabs/PersonUI"));
            personUi.GetComponent<PersonUI>().Person = person;
            personUi.SetParent(scrollContent, false);
            var avatarTransform = (RectTransform)personUi.FindChild("AvatarBackground");
            avatarTransform.GetComponent<Image>().color = person.Color;
            avatarTransform.FindChild("AvatarIcon").GetComponent<Image>().sprite = person.Avatar;
            personUi.FindChild("PersonName").GetComponent<Text>().text = person.Name;
            personUi.localPosition = new Vector2(i * (avatarWidth + avatarSpace), 0);
        }
    }

    public void AddNewMember()
    {
        var personEditMenu = Instantiate<RectTransform>(Resources.Load<RectTransform>("Prefabs/PersonEditMenu"));
        personEditMenu.SetParent(canvas, false);
    }

    public void DoneEditing()
    {
        if (persons.Count > 0)
        {
            SceneManager.LoadScene("FamilyViewMenu");
        }
    }
}
