﻿using System;
using System.Linq;
using Data;
using Manage.Services.Synchronizers;
using UnityEngine;
using UnityEngine.UI;

public class GoalMenu : MonoBehaviour
{
    public Image background;
    public GoalSynchronizer goalSynchronizer;
    public RectTransform content;
    public RewardPool rewardPool;
    public Sprite goalBackgroundSprite;

    public void Start()
    {
        var serverObject = GameObject.Find("Server");
        goalSynchronizer = serverObject.GetComponent<GoalSynchronizer>();
        OpenGoalMenu();
    }

    public void OpenGoalMenu()
    {
        // Select appropriate goals
        var goals = goalSynchronizer.Goals
            .Where((g => (!g.IsGroup.HasValue)
                || (g.IsGroup.Value && rewardPool.Person == null)
                || (!g.IsGroup.Value && rewardPool.Person != null)))
            .ToList();

        float? position = null;
        foreach (var goal in goals)
        {
            var go = new GameObject("Goal-" + goal.Id, typeof(Image), typeof(Button));
            var goalIcon = new GameObject("GoalIcon", typeof(Image));
            goalIcon.transform.SetParent(go.transform, false);
            goalIcon.transform.localScale = new Vector2(0.5f, 0.5f); // TODO: remove hardcode
            // Set position
            var goTransform = ((RectTransform)go.transform);
            var goalImage = goalIcon.GetComponent<Image>();
            var goalBackground = go.GetComponent<Image>();
            goTransform.SetParent(content, false);
            goTransform.anchorMin = new Vector2(0.5f, 0);
            goTransform.anchorMax = new Vector2(0.5f, 1);
            goTransform.offsetMin = new Vector2(goTransform.offsetMin.x, 0);
            goTransform.offsetMax = new Vector2(goTransform.offsetMax.x, 0);
            goTransform.pivot = new Vector2(goTransform.pivot.x, 1);

            var width = goTransform.sizeDelta.x;
            if (position == null)
                position = -(width * goals.Count / 2f) + width / 2f;
            goTransform.localPosition = new Vector2(position.Value, 0);
            position += width;
            content.sizeDelta += new Vector2(width, 0);

            // Set image and button
            goalImage.sprite = goal.Sprite;
            goalImage.preserveAspect = true;
            goalBackground.sprite = goalBackgroundSprite;
            goalBackground.preserveAspect = true;
            goalBackground.color = Constants.Colors.goalBackground;
            var goal1 = goal;
            go.GetComponent<Button>().onClick.AddListener(delegate { AssignGoal(goal1); });
        }
    }

    private void AssignGoal(Goal goal)
    {
        rewardPool.AssignGoal(goal);
        Destroy(gameObject);
    }

    public void Destroy()
    {
        Destroy(gameObject);
    }
}
