﻿using System;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.UI;

public class TaskBarUI : MonoBehaviour, IPointerDownHandler, IPointerUpHandler, IDragHandler
{
    public RectTransform content;
    public RectTransform canvas;
    public ScrollRect scrollRect;
    public static float scrollThreshold = 6.0f;
    public static int padding = 50;
    public static float canvasScale = 0.015f;

    private List<GameObject> tasks = new List<GameObject>();
    private TaskBallController selectedTask;
    private float? position;
    private bool isDraggingTask;
    private bool isScrollingEnabled;

    public void AddTask(GameObject task)
    {
        var radius = task.GetComponent<Collider2D>().bounds.extents.x;
        if (position == null)
        {
            position = 0;
            content.sizeDelta += new Vector2(padding * 2 - Constants.TaskBar.space / canvasScale, 0);
        }
        position += radius;
        tasks.Add(task);
        task.transform.SetParent(content, true);
        task.transform.localPosition = new Vector2(position.Value / canvasScale + padding, content.position.y);
        content.sizeDelta += new Vector2((radius * 2 + Constants.TaskBar.space) / canvasScale, 0);
        position += radius + Constants.TaskBar.space;
        if (content.sizeDelta.x > canvas.sizeDelta.x)
        {
            isScrollingEnabled = true;
            scrollRect.enabled = true;
        }

    }

    public int Remove(GameObject task)
    {
        int index = tasks.IndexOf(task);
        if (index >= 0)
        {
            tasks.RemoveAt(index);
            //if (tasks.Count > 0)
            //    UpdateTaskPositions(index);
        }
        return index;
    }

    public void Replace(GameObject oldTask, GameObject newTask)
    {
        tasks[tasks.IndexOf(oldTask)] = newTask;
    }

    public void OnPointerDown(PointerEventData eventData)
    {
        // Check if the pointer is over a task
        selectedTask = null;
        var colliders = Physics2D.OverlapPointAll(Camera.main.ScreenToWorldPoint(Input.mousePosition));
        foreach (var collider in colliders)
        {
            if ((selectedTask = collider.GetComponent<TaskBallController>()) != null)
                break;
        }
        if (selectedTask != null)
        {
            // The pointer is over a task, send it the OnPointerDown event
            eventData.selectedObject = selectedTask.gameObject;
            selectedTask.OnPointerDown(eventData);
        }
    }

    public void OnDrag(PointerEventData eventData)
    {
        if (selectedTask != null)
        {
            if (!isDraggingTask)
            {
                // Determine wether or not to scroll the task bar or drag task
                isDraggingTask = Mathf.Abs(eventData.delta.x) < scrollThreshold || !isScrollingEnabled;
                if (!isDraggingTask)
                    selectedTask = null; // Don't drag task, scroll instead
            }
            if (isDraggingTask)
            {
                // Drag task and disable the scrolling
                if (scrollRect.isActiveAndEnabled)
                    scrollRect.enabled = false;
                eventData.selectedObject = selectedTask.gameObject;
                selectedTask.OnDrag(eventData);
            }
        }
    }

    public void OnPointerUp(PointerEventData eventData)
    {
        if (selectedTask != null)
        {
            eventData.selectedObject = selectedTask.gameObject;
            selectedTask.OnPointerUp(eventData);
            selectedTask = null;
            isDraggingTask = false;
            scrollRect.enabled = isScrollingEnabled;
        }
    }
}
